import collections
from thingspace.models.cloud_file import CloudFile
from thingspace.packages.requests.requests.packages.urllib3.packages.six.moves import urllib

from thingspace.env import Env
from thingspace.exceptions import CloudError, NotFoundError
from thingspace.exceptions import OutOfSyncError
from thingspace.models.factories.FopsFactories import FopsFactories
from thingspace.packages.requests.requests import Request
from thingspace.packages.requests.requests.packages.urllib3.packages import six


class Fops():
    def metadata(self, path='/'):
        resp = self.networker(Request(
            'GET',
            str(Env.api_cloud + '/metadata' + path),
            headers={
                "Authorization": "Bearer " + self.access_token
            }
        ))

        if resp.status_code == 404:
            raise NotFoundError("path not found", response=resp)

        MetadataResponse = collections.namedtuple('MetadataResponse', 'files folders')

        json = resp.json()

        files = FopsFactories.files_from_json(self, json['folder'].get('file', []))
        folders = FopsFactories.folders_from_json(json['folder'].get('folder', []))

        return MetadataResponse(files, folders)


    def file_metadata(self, path):
        resp = self.networker(Request(
            'GET',
            str(Env.api_cloud + '/metadata' + path),
            headers={
                "Authorization": "Bearer " + self.access_token
            }
        ))

        if resp.status_code == 404:
            raise NotFoundError("path not found", response=resp)


        json = resp.json()

        file = FopsFactories.file_from_json(self, json['file'])

        return file

    def fullview(self, etag=None):
        if not self.authenticated:
            return None

        headers = {
            "Authorization": "Bearer " + self.access_token
        }

        if etag is not None:
            headers["X-Header-ETag"] = etag

        resp = self.networker(Request(
            'GET',
            Env.api_url + '/cloud/' + Env.api_version + '/fullview',
            headers=headers
        ))

        FullviewResponse = collections.namedtuple('FullviewResponse', 'files empty_folders etag deleted')

        # fullview is in sync, no changes
        if resp.status_code == 412:
            return FullviewResponse([], [], etag, [])

        # fullview is too far out of sync
        if resp.status_code == 205:
            raise OutOfSyncError("You are too far out of sync, please call fullview again with no etag", response=resp)

        json = resp.json()

        files = FopsFactories.files_from_json(self, json['data'].get('file', []))
        empty_folders = FopsFactories.folders_from_json(json['data'].get('folder', []))
        etag = resp.headers['X-Header-ETag']
        try:
            deleted = json['data']['deleted']['path']
        except KeyError:
            deleted = []

        return FullviewResponse(files, empty_folders, etag, deleted)

    def download_url(self, file_or_path):
        if isinstance(file_or_path, CloudFile):
            file_path = file_or_path.parent_path + '/' + file_or_path.name
        elif isinstance(file_or_path, six.string_types):
            file_path = file_or_path
        else:
            raise ValueError("file or path must be provided")

        req = Request(
            'GET',
            Env.api_cloud + '/files' + urllib.parse.quote(file_path),
            params={
                'access-token': self.access_token
            }
        )
        prepped = req.prepare()
        return prepped.url
